@@ -1,13 +1,19 @@
 /*
  * Copyright 2005-2008 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.196 2011/12/09 16:50:29 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.207 2011/12/19 04:39:24 vapier Exp $
  *
  * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2008 Mike Frysinger  - <vapier@gentoo.org>
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"  /* make sure we have EPREFIX, if set */
+#endif
+
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 #ifdef _AIX
 #define _LINUX_SOURCE_COMPAT
 #endif
@@ -48,7 +54,6 @@ int lookup_applet_idx(const char *);
 /* variables to control runtime behavior */
 char *module_name = NULL;
 char *modpath = NULL;
-char exact = 0;
 int found = 0;
 int verbose = 0;
 int quiet = 0;
@@ -60,6 +65,7 @@ static char *portarch;
 static char *portvdb;
 const char portcachedir[] = "metadata/cache";
 static char *portroot;
+static char *eprefix;
 static char *config_protect, *config_protect_mask;
 
 static char *pkgdir;
@@ -119,7 +125,7 @@ void no_colors()
 	NULL
 #define COMMON_GETOPTS_CASES(applet) \
 	case 'v': ++verbose; break; \
-	case 'q': ++quiet; if (freopen("/dev/null", "w", stderr)); break; \
+	case 'q': ++quiet; if (freopen("/dev/null", "w", stderr)) { /* ignore errors */ } break; \
 	case 'V': version_barf( applet ## _rcsid ); break; \
 	case 'h': applet ## _usage(EXIT_SUCCESS); break; \
 	case 'C': no_colors(); break; \
@@ -497,6 +503,17 @@ typedef struct {
 	const char *default_value;
 } env_vars;
 
+_q_static env_vars *get_portage_env_var(env_vars *vars, const char *name)
+{
+	size_t i;
+
+	for (i = 0; vars[i].name; ++i)
+		if (!strcmp(vars[i].name, name))
+			return &vars[i];
+
+	return NULL;
+}
+
 _q_static void set_portage_env_var(env_vars *var, const char *value)
 {
 	switch (var->type) {
@@ -669,12 +686,13 @@ void initialize_portage_env(void)
 		_Q_EVS(ISTR, CONFIG_PROTECT_MASK, config_protect_mask, "")
 		_Q_EVB(BOOL, NOCOLOR,             nocolor,             0)
 		_Q_EVS(ISTR, FEATURES,            features,            "noman noinfo nodoc")
+		_Q_EVS(STR,  EPREFIX,             eprefix,             EPREFIX)
 		_Q_EVS(STR,  PORTDIR,             portdir,             EPREFIX "usr/portage")
 		_Q_EVS(STR,  PORTAGE_BINHOST,     binhost,             DEFAULT_PORTAGE_BINHOST)
 		_Q_EVS(STR,  PORTAGE_TMPDIR,      port_tmpdir,         EPREFIX "var/tmp/portage/")
 		_Q_EVS(STR,  PKGDIR,              pkgdir,              EPREFIX "usr/portage/packages/")
 		_Q_EVS(STR,  Q_VDB,               portvdb,             EPREFIX "var/db/pkg")
-		{ }
+		{ NULL, 0, _Q_BOOL, { NULL }, 0, NULL, }
 
 #undef _Q_EV
 	};
@@ -705,7 +723,7 @@ void initialize_portage_env(void)
 			set_portage_env_var(var, s);
 		if (getenv("DEBUG") IF_DEBUG(|| 1)) {
 			fprintf(stderr, "%s = ", var->name);
-			switch (vars_to_read[i].type) {
+			switch (var->type) {
 			case _Q_BOOL: fprintf(stderr, "%i\n", *var->value.b); break;
 			case _Q_STR:
 			case _Q_ISTR: fprintf(stderr, "%s\n", *var->value.s); break;
@@ -713,6 +731,70 @@ void initialize_portage_env(void)
 		}
 	}
 
+	/* expand any nested variables e.g. PORTDIR=${EPREFIX}/usr/portage */
+	for (i = 0; vars_to_read[i].name; ++i) {
+		char *svar;
+
+		var = &vars_to_read[i];
+		if (var->type == _Q_BOOL)
+			continue;
+
+		while ((svar = strchr(*var->value.s, '$'))) {
+			env_vars *evar;
+			bool brace;
+			const char *sval;
+			size_t slen, pre_len, var_len, post_len;
+			char byte;
+
+			pre_len = svar - *var->value.s;
+
+			/* First skip the leading "${" */
+			s = ++svar;
+			brace = (*svar == '{');
+			if (brace)
+				s = ++svar;
+
+			/* Now skip the variable name itself */
+			while (isalnum(*svar) || *svar == '_')
+				++svar;
+
+			/* Finally skip the trailing "}" */
+			if (brace && *svar != '}') {
+				warn("invalid variable setting: %s\n", *var->value.s);
+				break;
+			}
+
+			var_len = svar - *var->value.s + 1;
+
+			byte = *svar;
+			*svar = '\0';
+			evar = get_portage_env_var(vars_to_read, s);
+			if (evar) {
+				sval = *evar->value.s;
+			} else {
+				sval = getenv(s);
+				if (!sval)
+					sval = "";
+			}
+			*svar = byte;
+			slen = strlen(sval);
+			post_len = strlen(svar + 1);
+			*var->value.s = xrealloc(*var->value.s, pre_len + MAX(var_len, slen) + post_len + 1);
+
+			/*
+			 * VAR=XxXxX	(slen = 5)
+			 * FOO${VAR}BAR
+			 * pre_len = 3
+			 * var_len = 6
+			 * post_len = 3
+			 */
+			memmove(*var->value.s + pre_len + slen,
+				*var->value.s + pre_len + var_len,
+				post_len + 1);
+			memcpy(*var->value.s + pre_len, sval, slen);
+		}
+	}
+
 	/* Make sure ROOT always ends in a slash */
 	var = &vars_to_read[0];
 	if ((*var->value.s)[var->value_len - 1] != '/') {
@@ -772,7 +854,7 @@ const char *initialize_flat(int cache_type)
 		goto ret;
 	}
 
-	if ((stat(cache_file, &st)) != (-1))
+	if (stat(cache_file, &st) != -1)
 		if (st.st_size == 0)
 			unlink(cache_file);
 
@@ -801,7 +883,7 @@ const char *initialize_flat(int cache_type)
 
 	gettimeofday(&start, NULL);
 
-	if ((a = scandir(".", &category, filter_hidden, alphasort)) < 0)
+	if ((a = scandir(".", &category, q_vdb_filter_cat, alphasort)) < 0)
 		goto ret;
 
 	for (i = 0; i < a; i++) {
@@ -812,7 +894,7 @@ const char *initialize_flat(int cache_type)
 			if ((strncmp(category[i]->d_name, "virtual", 7)) != 0)
 				continue;
 
-		if ((b = scandir(category[i]->d_name, &pn, filter_hidden, alphasort)) < 0)
+		if ((b = scandir(category[i]->d_name, &pn, q_vdb_filter_pkg, alphasort)) < 0)
 			continue;
 		for (c = 0; c < b; c++) {
 			char de[_Q_PATH_MAX];
@@ -845,12 +927,10 @@ const char *initialize_flat(int cache_type)
 			while (d--) free(eb[d]);
 			free(eb);
 		}
-		while (b--) free(pn[b]);
-		free(pn);
+		scandir_free(pn, b);
 	}
 	fclose(fp);
-	while (a--) free(category[a]);
-	free(category);
+	scandir_free(category, a);
 
 	if (quiet) goto ret;
 
@@ -1024,15 +1104,15 @@ static char *grab_vdb_item(const char *item, const char *CATEGORY, const char *P
 	return buf;
 }
 
-queue *get_vdb_atoms(int fullcpv);
-queue *get_vdb_atoms(int fullcpv)
+_q_static queue *get_vdb_atoms(int fullcpv)
 {
+	q_vdb_ctx *ctx;
+
 	int cfd, j;
 	int dfd, i;
 
 	char buf[_Q_PATH_MAX];
 	char slot[_Q_PATH_MAX];
-	static char savecwd[_POSIX_PATH_MAX];
 
 	struct dirent **cat;
 	struct dirent **pf;
@@ -1040,33 +1120,18 @@ queue *get_vdb_atoms(int fullcpv)
 	depend_atom *atom = NULL;
 	queue *cpf = NULL;
 
-	xgetcwd(savecwd, sizeof(savecwd));
+	ctx = q_vdb_open();
+	if (!ctx)
+		return NULL;
 
-	xchdir(savecwd);
-
-	if (chdir(portroot) != 0)
-		goto fuckit;
-
-	if (chdir(portvdb) != 0)
-		goto fuckit;
-
-	memset(buf, 0, sizeof(buf));
 	/* scan the cat first */
-	if ((cfd = scandir(".", &cat, filter_hidden, alphasort)) < 0)
+	if ((cfd = scandirat(ctx->vdb_fd, ".", &cat, q_vdb_filter_cat, alphasort)) < 0)
 		goto fuckit;
 
 	for (j = 0; j < cfd; j++) {
-		if (cat[j]->d_name[0] == '-')
+		if ((dfd = scandirat(ctx->vdb_fd, cat[j]->d_name, &pf, q_vdb_filter_pkg, alphasort)) < 0)
 			continue;
-		if (chdir(cat[j]->d_name) != 0)
-			continue;
-		if ((dfd = scandir(".", &pf, filter_hidden, alphasort)) < 0) {
-			xchdir("..");
-			continue;
-		}
 		for (i = 0; i < dfd; i++) {
-			if (pf[i]->d_name[0] == '-')
-				continue;
 			snprintf(buf, sizeof(buf), "%s/%s", cat[j]->d_name, pf[i]->d_name);
 			if ((atom = atom_explode(buf)) == NULL)
 				continue;
@@ -1074,16 +1139,12 @@ queue *get_vdb_atoms(int fullcpv)
 			slot[0] = '0';
 			slot[1] = 0;
 			strncat(buf, "/SLOT", sizeof(buf));
-			if (access(buf, R_OK) == 0) {
-				eat_file(buf, buf, sizeof(buf));
-				rmspace(buf);
-				if (strcmp(buf, "0") != 0)
-					strncpy(slot, buf, sizeof(slot));
-			}
+			eat_file_at(ctx->vdb_fd, buf, buf, sizeof(buf));
+			rmspace(buf);
 
 			if (fullcpv) {
 				if (atom->PR_int)
-					snprintf(buf, sizeof(buf), "%s/%s-%s-r%i", atom->CATEGORY, atom->PN, atom->PV , atom->PR_int);
+					snprintf(buf, sizeof(buf), "%s/%s-%s-r%i", atom->CATEGORY, atom->PN, atom->PV, atom->PR_int);
 				else
 					snprintf(buf, sizeof(buf), "%s/%s-%s", atom->CATEGORY, atom->PN, atom->PV);
 			} else {
@@ -1092,21 +1153,16 @@ queue *get_vdb_atoms(int fullcpv)
 			atom_implode(atom);
 			cpf = add_set(buf, slot, cpf);
 		}
-		xchdir("..");
-		while (dfd--) free(pf[dfd]);
-		free(pf);
+		scandir_free(pf, dfd);
 	}
+	scandir_free(cat, cfd);
 
-	/* cleanup */
-	while (cfd--) free(cat[cfd]);
-	free(cat);
-
-fuckit:
-	xchdir(savecwd);
+ fuckit:
+	q_vdb_close(ctx);
 	return cpf;
 }
 
-void cleanup()
+void cleanup(void)
 {
 	reinitialize_as_needed();
 	free_sets(virtuals);
@@ -1125,12 +1181,12 @@ int main(int argc, char **argv)
 	textdomain(argv0);
 #endif
 #if 1
-	if (fstat(fileno(stdout), &st) != (-1))
+	if (fstat(fileno(stdout), &st) != -1)
 		if (!isatty(fileno(stdout)))
 			if ((S_ISFIFO(st.st_mode)) == 0)
 				no_colors();
 #endif
-	if ((getenv("TERM") == NULL) || ((strcmp(getenv("TERM"), "dumb")) == 0))
+	if ((getenv("TERM") == NULL) || (strcmp(getenv("TERM"), "dumb") == 0))
 		no_colors();
 
 	initialize_portage_env();
